# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class EvmSwapsNetwork(str, Enum):
    """
    The network on which to perform the swap.
    """

    """
    allowed enum values
    """
    BASE = 'base'
    ETHEREUM = 'ethereum'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of EvmSwapsNetwork from a JSON string"""
        return cls(json.loads(json_str))


