# coding: utf-8

"""
    Coinbase Developer Platform APIs

    The Coinbase Developer Platform APIs - leading the world's transition onchain.

    The version of the OpenAPI document: 2.0.0
    Contact: cdp@coinbase.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from cdp.openapi_client.models.sign_evm_transaction_rule import SignEvmTransactionRule

class TestSignEvmTransactionRule(unittest.TestCase):
    """SignEvmTransactionRule unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> SignEvmTransactionRule:
        """Test SignEvmTransactionRule
            include_optional is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `SignEvmTransactionRule`
        """
        model = SignEvmTransactionRule()
        if include_optional:
            return SignEvmTransactionRule(
                action = 'accept',
                operation = 'signEvmTransaction',
                criteria = [{"type":"ethValue","ethValue":"1000000","operator":">="},{"type":"evmAddress","addresses":["0x742d35Cc6634C0532925a3b844Bc454e4438f44e"],"operator":"in"}]
            )
        else:
            return SignEvmTransactionRule(
                action = 'accept',
                operation = 'signEvmTransaction',
                criteria = [{"type":"ethValue","ethValue":"1000000","operator":">="},{"type":"evmAddress","addresses":["0x742d35Cc6634C0532925a3b844Bc454e4438f44e"],"operator":"in"}],
        )
        """

    def testSignEvmTransactionRule(self):
        """Test SignEvmTransactionRule"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
